"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EC2Wrapper = void 0;
const aws_sdk_1 = require("aws-sdk");
const utils_1 = require("../utils");
class EC2Wrapper {
    constructor(credentials) {
        this.ec2 = new aws_sdk_1.EC2(credentials);
    }
    /**
     *  Returns the promise that contains the vpc-id
     * @param {string} vpcName
     * @returns {Promise.<string>}
     */
    getVpcId(vpcName) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                Filters: [{
                        Name: "tag:Name",
                        Values: [vpcName]
                    }]
            };
            const vpcItems = yield utils_1.getAWSPagedResults(this.ec2, "describeVpcs", "Vpcs", "NextToken", "NextToken", params);
            if (vpcItems.length === 0) {
                throw new Error(`VPC with tag key 'Name' and tag value '${vpcName}' does not exist.`);
            }
            return vpcItems[0].VpcId;
        });
    }
    /**
     * Returns the promise that contains the subnet IDs
     *
     * @param {string} vpcId
     * @param {string} tagKey
     * @param {string[]} tagValues
     * @returns {Promise.<string[]>}
     */
    getSubnetIds(vpcId, tagKey, tagValues) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = {
                Filters: [{
                        Name: "vpc-id",
                        Values: [vpcId]
                    }, {
                        Name: `tag:${tagKey}`,
                        Values: tagValues
                    }]
            };
            const subnets = yield utils_1.getAWSPagedResults(this.ec2, "describeSubnets", "Subnets", "NextToken", "NextToken", params);
            const missingSubnetValues = tagValues.filter((tagValue) => {
                // collect subnets by name
                const subnetsByName = subnets.filter((subnet) => {
                    return utils_1.wildcardMatches(tagValue, utils_1.getValueFromTags(subnet.Tags, tagKey));
                });
                return subnetsByName.length === 0;
            });
            if (!subnets.length || missingSubnetValues.length) {
                throw new Error(`Subnets with vpc id '${vpcId}', tag key '${tagKey}' and tag values '${missingSubnetValues}' do not exist. ` +
                    "Please check the `tagKey` and `tagValues` are correct or remove it.");
            }
            return subnets.map((subnet) => subnet.SubnetId);
        });
    }
    /**
     *  Returns the promise that contains the security group IDs
     * @param {string} vpcId
     * @param {string[]} names
     * @param {string[]} tagKey
     * @param {string[]} tagValues
     * @returns {Promise.<string[]>}
     */
    getSecurityGroupIds(vpcId, names, tagKey, tagValues) {
        return __awaiter(this, void 0, void 0, function* () {
            // init filter by vpc id
            const params = { Filters: [{ Name: "vpc-id", Values: [vpcId] }] };
            // update filters with names if specified
            if (names) {
                params.Filters.push({ Name: "group-name", Values: names });
            }
            // update filters with tag and values if specified
            if (tagKey && tagValues) {
                params.Filters.push({ Name: `tag:${tagKey}`, Values: tagValues });
            }
            const securityGroups = yield utils_1.getAWSPagedResults(this.ec2, "describeSecurityGroups", "SecurityGroups", "NextToken", "NextToken", params);
            if (securityGroups.length === 0) {
                const namesErrorText = names ? `, names '${names}'` : "";
                const tagErrorText = tagKey && tagValues ? `, tag key '${tagKey}' and tag values '${tagValues}'` : "";
                throw new Error(`Security groups with vpc id '${vpcId}'${namesErrorText}${tagErrorText} do not exist`);
            }
            if (names) {
                const missingGroupsNames = names.filter((groupName) => {
                    // collect security groups by name
                    const securityGroupsByName = securityGroups.filter((securityGroup) => {
                        return utils_1.wildcardMatches(groupName, securityGroup.GroupName);
                    });
                    return securityGroupsByName.length === 0;
                });
                if (missingGroupsNames.length) {
                    throw new Error(`Security groups do not exist for the names: ${missingGroupsNames}. ` +
                        "Please check the 'names' are correct or remove it.");
                }
            }
            if (tagKey && tagValues) {
                tagValues = tagValues || [];
                const missingGroupsTagNames = tagValues.filter((tagValue) => {
                    // collect subnets by name
                    const groupsByName = securityGroups.filter((securityGroup) => {
                        const groupTagValue = utils_1.getValueFromTags(securityGroup.Tags, tagKey);
                        return groupTagValue === tagValue;
                    });
                    return groupsByName.length === 0;
                });
                if (missingGroupsTagNames.length) {
                    throw new Error(`Security groups do not exist for the tag '${tagKey}' and tag values: '${missingGroupsTagNames}'. ` +
                        "Please check the 'tagKey' and 'tagValues' are correct or remove it.");
                }
            }
            return securityGroups.map((group) => group.GroupId);
        });
    }
}
exports.EC2Wrapper = EC2Wrapper;
